Unit ClassVlakna;

Interface

Uses Sorting, Windows, ComCtrls;

Const
      VLAKNA_SLOUPEC_TID         = 0;
      VLAKNA_SLOUPEC_PRIORITY    = 1;
      VLAKNA_SLOUPEC_SUSPENDED   = 2;
      VLAKNA_MAX_SLOUPCU         = 3;

Type
     TVlakno = Record
                TID          : Cardinal;
                PID          : Cardinal;
                Priorita     : Integer;
                SuspendCount : Integer;
               end;
     PVlakno = ^TVlakno;
     TThreadSortStyle = (tssID, tssPriorita,
                         tssSuspendCount);

     TVlakna = Class (TSortList)
               Private
                FSortStyle : TThreadSortStyle;
                FPID       : Cardinal;
               Protected
                Procedure SetSortStyle(H:TThreadSortStyle);               Virtual;
                Function  JeVetsi(A,B:Pointer; K:Cardinal):Boolean; Override;
                Function  JeRovno(A,B:Pointer; K:Cardinal):Boolean; Override;
               Public
                Constructor Create(AListView:TListView; APID:Cardinal); Reintroduce; Virtual;
                Destructor  Destroy; Override;
                Function GetThread(Index:Integer):PVlakno;
                Procedure Enumerate; Virtual;
                Procedure   ListViewVykresliJednoho(Sender:TObject; Item:TListItem);   Override;
                Procedure   ListViewColumnClick(Sender: TObject; Column: TListColumn); Override;
                Function    Open(Index:Integer; DesiredAccess:Cardinal):THandle;
                Function    Suspend(Index:Integer):Cardinal;
                Function    Resume(Index:Integer):Cardinal;
                Function    Terminate(Index:Integer):Cardinal;
                Property  SortStyle : TThreadSortStyle Read FSortStyle Write SetSortStyle;
                Property  PID : Cardinal Read FPID;
               end;

Implementation

Uses TlHelp32, OSVersion,
     SysUtils,
     TProcImport,
     Forms,
     ProceduresImport;

Const
      VLAKNA_KRITERIUM_TID          = 0;
      VLAKNA_KRITERIUM_PRIORITA     = 1;
      VLAKNA_KRITERIUM_SUSPENDCOUNT = 2;

Function  TVlakna.JeVetsi(A,B:Pointer; K:Cardinal):Boolean;
Var X, Y : PVlakno;
begin
X:=A;
Y:=B;
Case K Of
     VLAKNA_KRITERIUM_TID          : Result:=X.TID > Y.TID;
     VLAKNA_KRITERIUM_PRIORITA     : Result:=X.Priorita > Y.Priorita;
     VLAKNA_KRITERIUM_SUSPENDCOUNT : Result:=X.SuspendCount > Y.SuspendCount;
     end;
end;

Function TVlakna.JeRovno(A,B:Pointer; K:Cardinal):Boolean;
Var X, Y : PVlakno;
begin
X:=A;
Y:=B;
Case K Of
     VLAKNA_KRITERIUM_TID           : Result:=X.TID = Y.TID;
     VLAKNA_KRITERIUM_PRIORITA      : Result:=X.Priorita = Y.Priorita;
     VLAKNA_KRITERIUM_SUSPENDCOUNT  : Result:=X.SuspendCount = Y.SuspendCount;
     end;
end;

Procedure TVlakna.SetSortStyle(H:TThreadSortStyle);
begin
If H <> FSortStyle Then
   begin
   Opacne:=False;
   If Kriteria.Count > 1 Then
      Kriteria.Delete(0);
   FSortStyle:=H;
   Case H Of
        tssPriorita     : Kriteria.Insert(0,Pointer(VLAKNA_KRITERIUM_PRIORITA));
        tssSuspendCount : Kriteria.Insert(0,Pointer(VLAKNA_KRITERIUM_SUSPENDCOUNT));
        end;
   end
Else Opacne:= Not Opacne;
Serad;
end;

Constructor TVlakna.Create(AListView:TListView; APID:Cardinal);
begin
Inherited Create(AListView);
Opacne:=False;
FPID:=APID;
Kriteria.Add(Pointer(VLAKNA_KRITERIUM_TID));
FJmena.LoadFromFile(ExtractFilePath(Application.ExeName)+'Data\thread.col');
FSortStyle:=tssID;
PocetSloupcu:=VLAKNA_MAX_SLOUPCU;
end;

Destructor TVlakna.Destroy;
begin
Clear;
Inherited Destroy;
end;

Function TVlakna.GetThread(Index:Integer):PVlakno;
begin
Result:=Items[Index];
end;

Procedure TVlakna.Enumerate;
Var hSnap   : THandle;
    TE      : ThreadEntry32;
    Item    : PVlakno;
    hThread : THandle;
begin
Clear;
If Win9xMe Or WinXP Then
   begin
   hSnap:=ThreadSearchInit;
   If hSnap > 0 Then
      begin
      TE.dwSize:=SizeOF(TE);
      If ThreadSearchFirst(hSnap,TE) Then
         begin
         Repeat
         If TE.th32OwnerProcessID = PID Then
            begin
            New(Item);
            Item.TID:=TE.th32ThreadID;
            Item.PID:=TE.th32OwnerProcessID;
            Item.Priorita:=TE.tpBasePri;
            Item.SuspendCount:=-1;
            Add(Item);
            hThread:=Open(Count-1,THREAD_SUSPEND_RESUME Or THREAD_QUERY_INFORMATION);
            If hThread > 0 Then
               begin
               Item.Priorita:=GetThreadPriority(hThread);
               If Item.TID <> GetCurrentThreadID Then
                  begin
                  Item.SuspendCount:=SuspendThread(hThread);
                  ResumeThread(hThread);
                  end
               Else Item.SuspendCount:=0;
               CloseHandle(hThread);
               end;
            end;
         Until Not ThreadSearchNext(hSnap,TE);
         end
      Else ErrorMessage('ThreadSearchFirst: '+SysErrorMessage(GetLastError));
      ThreadSearchDestroy(hSnap);
      end
   Else ErrorMessage('ThreadSearchInit: '+SysErrorMessage(GetLastError));
   end;
Serad;
If Assigned(FListView) Then
   begin
   FListView.Invalidate;
   FListView.Items.Count:=Count;
   end;
end;

Function  TVlakna.Open(Index:Integer; DesiredAccess:Cardinal):THandle;
Var TID : Cardinal;
begin
TID:=GetThread(Index).TID;
Result:=UniversalOpenThread(DesiredAccess,False,TID);
end;

Procedure TVlakna.ListViewVykresliJednoho(Sender:TObject; Item:TListItem);
Var T : PVlakno;
    S : String;
begin
T:=GetThread(Item.Index);
With Item Do
     begin
     If SloupecDejStav(VLAKNA_SLOUPEC_TID) Then
        Caption:=IntToHex(Int64(T.TID),8);
     If SloupecDejStav(VLAKNA_SLOUPEC_PRIORITY) Then
        begin
        S:=IntToStr(T.Priorita)+' [';
        Case T.Priorita Of
             THREAD_PRIORITY_ABOVE_NORMAL  : S:=S+'Nadprmrn';
             THREAD_PRIORITY_BELOW_NORMAL  : S:=S+'Podprmrn';
             THREAD_PRIORITY_HIGHEST       : S:=S+'Vysok';
             THREAD_PRIORITY_IDLE          : S:=S+'Mal';
             THREAD_PRIORITY_LOWEST        : S:=S+'Nzk';
             THREAD_PRIORITY_NORMAL        : S:=S+'Normln';
             THREAD_PRIORITY_TIME_CRITICAL : S:=S+'Reln as';
             end;
        S:=S+']';
        SubItems.Add(S);
        end;
     If SloupecDejStav(VLAKNA_SLOUPEC_SUSPENDED) Then
        begin
        S:=IntToStr(T.SuspendCount)+' [';
        If T.SuspendCount > 0 Then
           S:=S+' Pozastaveno'
        Else S:=S+' b';
        S:=S+']';
        SubItems.Add(S);
        end;
     end;
end;

Procedure TVlakna.ListViewColumnClick(Sender: TObject; Column: TListColumn);
begin
Enumerate;
SortStyle:=TThreadSortStyle(Column.Tag);
end;

Function TVlakna.Suspend(Index:Integer):Cardinal;
Var X : Cardinal;
begin
Result:=Open(Index,THREAD_SUSPEND_RESUME);
If Result > 0 Then
   begin
   X:=SuspendThread(Result);
   CloseHandle(Result);
   Result:=X;
   end;
Enumerate;
end;

Function TVlakna.Resume(Index:Integer):Cardinal;
Var X : Cardinal;
begin
Result:=Open(Index,THREAD_SUSPEND_RESUME);
If Result > 0 Then
   begin
   X:=ResumeThread(Result);
   CloseHandle(Result);
   Result:=X;
   end;
Enumerate;
end;

Function TVlakna.Terminate(Index:Integer):Cardinal;
Var X : Cardinal;
begin
Result:=Open(Index,THREAD_TERMINATE);
If Result > 0 Then
   begin
   TerminateThread(Result,0);
   WaitForSingleObject(Result,3000);
   GetExitCodeThread(Result,X);
   CloseHandle(Result);
   Result:=X;
   end;
Sleep(200);
Enumerate;
end;

End.
