Unit DriverManager;

Interface

Uses WinSvc;

Type
      TDriverManager = Class
                       Private
                        FhSCM    : THandle;
                        FhDriver : THandle;
                        FDriverName : String;
                        Function    Open(DriverName:String):Cardinal;
                        Function    Close:Cardinal;
                       Public
                        Constructor Create;
                        Function    Install(FileName:String; DisplayName:String):Cardinal;
                        Function    UnInstall(DriverName:String):Cardinal;
                        Function    Start(DriverName:String):Cardinal;
                        Function    Stop(DriverName:String):Cardinal;
                        Destructor  Destroy; Override;
                       end;

Implementation

Uses SysUtils;

Constructor TDriverManager.Create;
begin
Inherited Create;
FhSCM:=OpenSCManager(Nil,Nil,SC_MANAGER_ALL_ACCESS);
If FhSCM = 0 Then
   Raise Exception.Create(SysErrorMessage(GetLastError));
end;

Destructor TDriverManager.Destroy;
begin
CloseServiceHandle(FhSCM);
Inherited Destroy;
end;

Function    TDriverManager.Install(FileName:String; DisplayName:String):Cardinal;
begin
Result:=0;
FhDriver:=CreateService(FhSCM,
                        PAnsiChar(DisplayName),
                        PAnsiChar(DisplayName),
                        SERVICE_ALL_ACCESS,
                        SERVICE_KERNEL_DRIVER,
                        SERVICE_DEMAND_START,
                        SERVICE_ERROR_NORMAL,
                        PAnsiChar(FileName),
                        Nil,
                        Nil,
                        Nil,
                        Nil,
                        Nil);
If FhDriver = 0 Then
   Result:=GetLastError;
Close;
end;

Function TDriverManager.UnInstall(DriverName:String):Cardinal;
begin
Result:=Open(DriverName);
If Result = 0 Then
   begin
   If Not DeleteService(FhDriver) Then
     Result:=GetLastError;
   Close;
   end;
end;

Function    TDriverManager.Start(DriverName:String):Cardinal;
var v : PAnsiChar;
begin
V:=Nil;
Result:=Open(DriverName);
If Result = 0 Then
   begin
   If Not StartService(FhDriver,0,v) Then
      Result:=GetLastError;
   Close;
   end
end;

Function    TDriverManager.Stop(DriverName:String):Cardinal;
Var S : _SERVICE_STATUS;
begin
Result:=Open(DriverName);
If Result = 0 Then
   begin
   If Not ControlService(FhDriver,SERVICE_CONTROL_STOP,S) Then
      Result:=GetLastError;
   Close;
   end;
end;

Function  TDriverManager.Open(DriverName:String):Cardinal;
begin
Result:=0;
FhDriver:=OpenService(FhSCM,PAnsiChar(DriverName),SERVICE_ALL_ACCESS);
If FhDriver = 0 Then
   Result:=GetLastError;
end;

Function TDriverManager.Close:Cardinal;
begin
CloseServiceHandle(FhDriver);
end;

End.
