Unit WinNTFunctions;

Interface

Uses TlHelp32, Windows;

Const
    STATUS_SUCCESS = $00000000;
    STATUS_UNSUCCESSFUL           = $C0000001;
    STATUS_NOT_IMPLEMENTED        = $C0000002;
    STATUS_INVALID_INFO_CLASS     = $C0000003;
    STATUS_INFO_LENGTH_MISMATCH   = $C0000004;
    STATUS_ACCESS_VIOLATION       = $C0000005;
    STATUS_IN_PAGE_ERROR          = $C0000006;
    STATUS_PAGEFILE_QUOTA         = $C0000007;
    STATUS_INVALID_HANDLE         = $C0000008;
    STATUS_BAD_INTERNAL_STACK     = $C0000009;
    STATUS_BAD_INTERNAL_PC        = $C000000A;
    STATUS_INVALID_CID            = $C000000B;
    STATUS_TIMER_NOT_CANCELED     = $C000000C;
    STATUS_INVALID_PARAMETER      = $C000000D;
    STATUS_NO_SUCH_DEVICE         = $C000000E;
    STATUS_NO_SUCH_FILE           = $C000000F;
    STATUS_INVALID_DEVICE_REQUEST = $C0000010;

    // System information classes
    SystemBasicInformation = 0;
    SystemProcessorInformation = 1;
    SystemPerformanceInformation = 2;
    SystemTimeOfDayInformation = 3;
    SystemNotImplemented1 = 4;
    SystemProcessesAndThreadsInformation = 5;
    SystemCallCounts = 6;
    SystemConfigurationInformation = 7;
    SystemProcessorTimes = 8;
    SystemGlobalFlag = 9;
    SystemNotImplemented2 = 10;
    SystemModuleInformation = 11;
    SystemLockInformation = 12;
    SystemNotImplemented3 = 13;
    SystemNotImplemented4 = 14;
    SystemNotImplemented5 = 15;
    SystemHandleInformation = 16;
    SystemObjectInformation = 17;
    SystemPagefileInformation = 18;
    SystemInstructionEmulationCounts = 19;
    SystemInvalidInfoClass1 = 20;
    SystemCacheInformation = 21;
    SystemPoolTagInformation = 22;
    SystemProcessorStatistics = 23;
    SystemDpcInformation = 24;
    SystemNotImplemented6 = 25;
    SystemLoadImage = 26;
    SystemUnloadImage = 27;
    SystemTimeAdjustment = 28;
    SystemNotImplemented7 = 29;
    SystemNotImplemented8 = 30;
    SystemNotImplemented9 = 31;
    SystemCrashDumpInformation = 32;
    SystemExceptionInformation = 33;
    SystemCrashDumpStateInformation = 34;
    SystemKernelDebuggerInformation = 35;
    SystemContextSwitchInformation = 36;
    SystemRegistryQuotaInformation = 37;
    SystemLoadAndCallImage = 38;
    SystemPrioritySeparation = 39;
    SystemNotImplemented10 = 40;
    SystemNotImplemented11 = 41;
    SystemInvalidInfoClass2 = 42;
    SystemInvalidInfoClass3 = 43;
    SystemTimeZoneInformation = 44;
    SystemLookasideInformation = 45;
    SystemSetTimeSlipEvent = 46;
    SystemCreateSession = 47;
    SystemDeleteSession = 48;
    SystemInvalidInfoClass4 = 49;
    SystemRangeStartInformation = 50;
    SystemVerifierInformation = 51;
    SystemAddVerifier = 52;
    SystemSessionProcessesInformation = 53;

    // Thread information classes
    ThreadBasicInformation = 0;

    // Process information class
    ProcessBasicInformation = 0;
    ProcessQuotaLimits      = 1;
    ProcessIoCounters       = 2;
    ProcessVmCounters       = 3;
    ProcessTimes            = 4;
    ProcessBasePriority     = 5;
    ProcessRaisePriority    = 6;
    ProcessDebugPort        = 7;
    ProcessExceptionPort    = 8;
    ProcessAccessToken      = 9;
    ProcessLdtInformation   = 10;
    ProcessLdtSize          = 11;
    ProcessDefaultHardErrorMode = 12;
    ProcessIoPortHandlers = 13;          // Note: this is kernel mode only
    ProcessPooledUsageAndLimits = 14;
    ProcessWorkingSetWatch = 15;
    ProcessUserModeIOPL = 16;
    ProcessEnableAlignmentFaultFixup = 17;
    ProcessPriorityClass = 18;
    ProcessWx86Information = 19;
    ProcessHandleCount = 20;
    ProcessAffinityMask = 21;
    ProcessPriorityBoost = 22;
    ProcessDeviceMap = 23;
    ProcessSessionInformation = 24;
    ProcessForegroundInformation = 25;
    ProcessWow64Information = 26;

Type
  // basics
  PVOID = Pointer;
  LONG  = Integer;
  HANDLE = THandle;
  UCHAR = {$IFDEF USE_DELPHI_TYPES}Windows.UCHAR{$ELSE}Char{$ENDIF};
  USHORT = Word;
  ULONG = {$IFDEF USE_DELPHI_TYPES}Windows.ULONG{$ELSE}Longword{$ENDIF};
  PWSTR = {$IFDEF USE_DELPHI_TYPES}Windows.LPWSTR{$ELSE}^WCHAR{$ENDIF};


  PUNICODE_STRING = ^UNICODE_STRING;
  _UNICODE_STRING = record
    Length: USHORT;
    MaximumLength: USHORT;
    Buffer: PWSTR;
  end;
  UNICODE_STRING = _UNICODE_STRING;

  POOL_TYPE = DWORD;
  NTSTATUS = DWORD;

  _OBJECT_INFORMATION_CLASS = (
    ObjectBasicInformation,
    ObjectNameInformation,
    ObjectTypeInformation,
    ObjectAllTypesInformation,
    ObjectHandleInformation);
  OBJECT_INFORMATION_CLASS = _OBJECT_INFORMATION_CLASS;

  _OBJECT_BASIC_INFORMATION = record // Information Class 0
    Attributes: ULONG;
    GrantedAccess: ACCESS_MASK;
    HandleCount: ULONG;
    PointerCount: ULONG;
    PagedPoolUsage: ULONG;
    NonPagedPoolUsage: ULONG;
    Reserved: array [0..2] of ULONG;
    NameInformationLength: ULONG;
    TypeInformationLength: ULONG;
    SecurityDescriptorLength: ULONG;
    CreateTime: LARGE_INTEGER;
  end;
  OBJECT_BASIC_INFORMATION = _OBJECT_BASIC_INFORMATION;
  POBJECT_BASIC_INFORMATION = ^OBJECT_BASIC_INFORMATION;

  _OBJECT_NAME_INFORMATION = record // Information Class 1
    Name: UNICODE_STRING;
  end;
  OBJECT_NAME_INFORMATION = _OBJECT_NAME_INFORMATION;
  POBJECT_NAME_INFORMATION = ^OBJECT_NAME_INFORMATION;

  _OBJECT_TYPE_INFORMATION = record // Information Class 2
    Name: UNICODE_STRING;
    ObjectCount: ULONG;
    HandleCount: ULONG;
    Reserved1: array [0..3] of ULONG;
    PeakObjectCount: ULONG;
    PeakHandleCount: ULONG;
    Reserved2: array [0..3] of ULONG;
    InvalidAttributes: ULONG;
    GenericMapping: GENERIC_MAPPING;
    ValidAccess: ULONG;
    Unknown: UCHAR;
    MaintainHandleDatabase: ByteBool;
    Reserved3: array [0..1] of UCHAR;
    PoolType: POOL_TYPE;
    PagedPoolUsage: ULONG;
    NonPagedPoolUsage: ULONG;
  end;
  OBJECT_TYPE_INFORMATION = _OBJECT_TYPE_INFORMATION;
  POBJECT_TYPE_INFORMATION = ^OBJECT_TYPE_INFORMATION;

  _OBJECT_ALL_TYPES_INFORMATION = record // Information Class 3
    NumberOfTypes: ULONG;
    TypeInformation: OBJECT_TYPE_INFORMATION;
  end;
  OBJECT_ALL_TYPES_INFORMATION = _OBJECT_ALL_TYPES_INFORMATION;
  POBJECT_ALL_TYPES_INFORMATION = ^OBJECT_ALL_TYPES_INFORMATION;

  _OBJECT_HANDLE_ATTRIBUTE_INFORMATION = record // Information Class 4
    Inherit: ByteBool;
    ProtectFromClose: ByteBool;
  end;
  OBJECT_HANDLE_ATTRIBUTE_INFORMATION = _OBJECT_HANDLE_ATTRIBUTE_INFORMATION;
  POBJECT_HANDLE_ATTRIBUTE_INFORMATION = ^OBJECT_HANDLE_ATTRIBUTE_INFORMATION;

  _SYSTEM_OBJECT_TYPE_INFORMATION = record // Information Class 17
    NextEntryOffset: ULONG;
    ObjectCount: ULONG;
    HandleCount: ULONG;
    TypeNumber: ULONG;
    InvalidAttributes: ULONG;
    GenericMapping: GENERIC_MAPPING;
    ValidAccessMask: ACCESS_MASK;
    PoolType: POOL_TYPE;
    Unknown: UCHAR;
    Name: UNICODE_STRING;
  end;

  SYSTEM_OBJECT_TYPE_INFORMATION = _SYSTEM_OBJECT_TYPE_INFORMATION;
  PSYSTEM_OBJECT_TYPE_INFORMATION = ^SYSTEM_OBJECT_TYPE_INFORMATION;

  _SYSTEM_OBJECT_INFORMATION = record
    NextEntryOffset: ULONG;
    Object_: PVOID;
    CreatorProcessId: ULONG;
    Unknown: USHORT;
    Flags: USHORT;
    PointerCount: ULONG;
    HandleCount: ULONG;
    PagedPoolUsage: ULONG;
    NonPagedPoolUsage: ULONG;
    ExclusiveProcessId: ULONG;
    SecurityDescriptor: PSECURITY_DESCRIPTOR;
    Name: UNICODE_STRING;
  end;
  SYSTEM_OBJECT_INFORMATION = _SYSTEM_OBJECT_INFORMATION;
  PSYSTEM_OBJECT_INFORMATION = ^SYSTEM_OBJECT_INFORMATION;

  _SYSTEM_HANDLE_TABLE_ENTRY_INFO = Record
    UniqueProcessId: ULONG;
    ObjectTypeIndex: UCHAR;
    HandleAttributes: UCHAR;
    HandleValue: USHORT;
    Object_: PVOID;
    GrantedAccess: ULONG;
  end;
  SYSTEM_HANDLE_TABLE_ENTRY_INFO = _SYSTEM_HANDLE_TABLE_ENTRY_INFO;
  PSYSTEM_HANDLE_TABLE_ENTRY_INFO = ^SYSTEM_HANDLE_TABLE_ENTRY_INFO;

  SYSTEM_HANDLE_INFORMATION = Record
                              NumberOfHandles: ULONG;
                              Handles: Array [0..1] Of SYSTEM_HANDLE_TABLE_ENTRY_INFO;
                              end;
  PSYSTEM_HANDLE_INFORMATION = ^SYSTEM_HANDLE_INFORMATION;

_SYSTEM_MODULE_INFORMATION = record // Information Class 11
    Reserved: array [0..1] of ULONG;
    Base: PVOID;
    Size: ULONG;
    Flags: ULONG;
    Index: USHORT;
    Unknown: USHORT;
    LoadCount: USHORT;
    ModuleNameOffset: USHORT;
    ImageName: Array [0..255] Of CHAR;
  end;
  
  SYSTEM_MODULE_INFORMATION = _SYSTEM_MODULE_INFORMATION;
  PSYSTEM_MODULE_INFORMATION = ^SYSTEM_MODULE_INFORMATION;

  _CLIENT_ID = Record
    UniqueProcess: HANDLE;
    UniqueThread: HANDLE;
    end;

  CLIENT_ID = _CLIENT_ID;
  PCLIENT_ID = ^CLIENT_ID;
  TClientID = CLIENT_ID;
  PClientID = ^TClientID;
  KAFFINITY = ULONG;
  PKAFFINITY = ^KAFFINITY;
  KPRIORITY = ULONG;

  _KWAIT_REASON = (
    Executive,
    FreePage,
    PageIn,
    PoolAllocation,
    DelayExecution,
    Suspended,
    UserRequest,
    WrExecutive,
    WrFreePage,
    WrPageIn,
    WrPoolAllocation,
    WrDelayExecution,
    WrSuspended,
    WrUserRequest,
    WrEventPair,
    WrQueue,
    WrLpcReceive,
    WrLpcReply,
    WrVirtualMemory,
    WrPageOut,
    WrRendezvous,
    Spare2,
    Spare3,
    Spare4,
    Spare5,
    Spare6,
    WrKernel,
    MaximumWaitReason);
  KWAIT_REASON = _KWAIT_REASON;

  _THREAD_BASIC_INFORMATION = record // Information Class 0
    ExitStatus: NTSTATUS;
    TebBaseAddress: PVOID;
    ClientId: CLIENT_ID;
    AffinityMask: KAFFINITY;
    Priority: KPRIORITY;
    BasePriority: KPRIORITY;
  end;
  THREAD_BASIC_INFORMATION = _THREAD_BASIC_INFORMATION;
  PTHREAD_BASIC_INFORMATION = ^THREAD_BASIC_INFORMATION;

  // Process
   _PROCESS_PRIORITY_CLASS = record // Information Class 18
    Foreground: ByteBool;
    PriorityClass: UCHAR;
  end;
  PROCESS_PRIORITY_CLASS = _PROCESS_PRIORITY_CLASS;
  PPROCESS_PRIORITY_CLASS = ^PROCESS_PRIORITY_CLASS;

  _PROCESS_PARAMETERS = record
    AllocationSize: ULONG;
    Size: ULONG;
    Flags: ULONG;
    Zero: ULONG;
    Console: LONG;
    ProcessGroup: ULONG;
    hStdInput: HANDLE;
    hStdOutput: HANDLE;
    hStdError: HANDLE;
    CurrentDirectoryName: UNICODE_STRING;
    CurrentDirectoryHandle: HANDLE;
    DllPath: UNICODE_STRING;
    ImageFile: UNICODE_STRING;
    CommandLine: UNICODE_STRING;
    Environment: PWSTR;
    dwX: ULONG;
    dwY: ULONG;
    dwXSize: ULONG;
    dwYSize: ULONG;
    dwXCountChars: ULONG;
    dwYCountChars: ULONG;
    dwFillAttribute: ULONG;
    dwFlags: ULONG;
    wShowWindow: ULONG;
    WindowTitle: UNICODE_STRING;
    Desktop: UNICODE_STRING;
    Reserved: UNICODE_STRING;
    Reserved2: UNICODE_STRING;
  end;
  PROCESS_PARAMETERS = _PROCESS_PARAMETERS;
  PPROCESS_PARAMETERS = ^PROCESS_PARAMETERS;
  PPPROCESS_PARAMETERS = ^PPROCESS_PARAMETERS;

  _PROCESS_BASIC_INFORMATION = Record
                               ExitStatus      : NTSTATUS;
                               PebBaseAddress  : PVOID;
                               AffinityMask    : KAFFINITY;
                               BasePriority    : KPRIORITY;
                               UniqueProcessID : ULONG;
                               ParentProcessID : ULONG;
                               end;
  PPROCESS_BASIC_INFORMATION = ^_PROCESS_BASIC_INFORMATION;
  PROCESS_BASIC_INFORMATION = _PROCESS_BASIC_INFORMATION;

Function NtQueryInformationProcess(ProcessHandle: HANDLE; ProcessInformationClass: Cardinal; ProcessInformation: PVOID; ProcessInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; stdcall;
Function NtQueryInformationThread(ThreadHandle: HANDLE; ThreadInformationClass: Cardinal; ThreadInformation: PVOID; ThreadInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; stdcall;
Function NtAlertThread(ThreadHandle: HANDLE): NTSTATUS; Stdcall;
Function NtAlertResumeThread(ThreadHandle: HANDLE; PreviousSuspendCount: PULONG): NTSTATUS; stdcall;
Function NtSetInformationThread(ThreadHandle: HANDLE; ThreadInformationClass: Cardinal; ThreadInformation: PVOID; ThreadInformationLength: ULONG): NTSTATUS; Stdcall;
Function NtQuerySystemInformation(SystemInformationClass:Cardinal; SystemInformation: PVOID; SystemInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; Stdcall;
Function WinNTProcessSearchInit:THandle;   StdCall;
Function WinNTProcessSearchFirst(hSearch:THandle; Var PE:ProcessEntry32):Boolean;
Function WinNTProcessSearchNext(hSearch:THandle; Var PE:ProcessEntry32):Boolean;                                         StdCall;
Procedure WinNTProcessSearchDestroy(hSearch:THandle); StdCall;
Function NtQueryObject(ObjectHandle: HANDLE; ObjectInformationClass: Cardinal; ObjectInformation: PVOID; ObjectInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; stdcall;
Function NtDuplicateObject(SourceProcessHandle: HANDLE; SourceHandle: HANDLE; TargetProcessHandle: HANDLE; TargetHandle: PHANDLE; DesiredAccess: ACCESS_MASK; Attributes: ULONG; Options: ULONG): NTSTATUS; stdcall;
Function NtClose(Handle: HANDLE): NTSTATUS; stdcall;
Function GetProcessID(hProcess:Cardinal):Cardinal;

Implementation

Uses SysUtils, ProceduresImport;

Type
    TNtQueryObject            = Function (ObjectHandle: HANDLE; ObjectInformationClass: Cardinal; ObjectInformation: PVOID; ObjectInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; stdcall;
    TNtQuerySystemInformation = Function (SystemInformationClass: Cardinal; SystemInformation: PVOID; SystemInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; stdcall;
    TNtDuplicateObject        = Function (SourceProcessHandle: HANDLE; SourceHandle: HANDLE; TargetProcessHandle: HANDLE; TargetHandle: PHANDLE; DesiredAccess: ACCESS_MASK; Attributes: ULONG; Options: ULONG): NTSTATUS; stdcall;
    TNtClose                  = Function (Handle: HANDLE): NTSTATUS; stdcall;
    TNtQueryInformationThread = Function (ThreadHandle: HANDLE; ThreadInformationClass: Cardinal; ThreadInformation: PVOID; ThreadInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; stdcall;
    TNtSetInformationThread = Function (ThreadHandle: HANDLE; ThreadInformationClass: Cardinal; ThreadInformation: PVOID; ThreadInformationLength: ULONG): NTSTATUS; stdcall;
    TNtAlertThread = Function (ThreadHandle: HANDLE): NTSTATUS; stdcall;
    TNtAlertResumeThread = Function (ThreadHandle: HANDLE; PreviousSuspendCount: PULONG): NTSTATUS; stdcall;
    TNtQueryInformationProcess = Function (ProcessHandle: HANDLE; ProcessInformationClass: Cardinal; ProcessInformation: PVOID; ProcessInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; stdcall;

Var
    _NtQueryObject : TNtQueryObject;
    _NtQuerySystemInformation : TNtQuerySystemInformation;
    _NtDuplicateObject : TNtDuplicateObject;
    _NtClose : TNtClose;
    _NtQueryInformationThread : TNtQueryInformationThread;
    _NtSetInformationThread : TNtSetInformationThread;
    _NtAlertThread : TNtAlertThread;
    _NtAlertResumeThread : TNtAlertResumeThread;
    _NtQueryInformationProcess : TNtQueryInformationProcess;

Function NtQuerySystemInformation(SystemInformationClass:Cardinal; SystemInformation: PVOID; SystemInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; stdcall;
begin
Result:=_NtQuerySystemInformation(SystemInformationClass,SystemInformation,SystemInformationLength,ReturnLength);
end;

Function NtQueryObject(ObjectHandle: HANDLE; ObjectInformationClass:Cardinal; ObjectInformation: PVOID; ObjectInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; stdcall;
begin
Result:=_NtQueryObject(ObjectHandle,ObjectInformationClass,ObjectInformation,ObjectInformationLength,ReturnLength);
end;

Function NtDuplicateObject(SourceProcessHandle: HANDLE; SourceHandle: HANDLE; TargetProcessHandle: HANDLE; TargetHandle: PHANDLE; DesiredAccess: ACCESS_MASK; Attributes: ULONG; Options: ULONG): NTSTATUS; stdcall;
begin
Result:=_NtDuplicateObject(SourceProcessHandle,SourceHandle,TargetProcessHandle,TargetHandle,DesiredAccess,Attributes,Options);
end;

Function NtClose(Handle: HANDLE): NTSTATUS; stdcall;
begin
Result:=_NtClose(Handle);
end;

Function WinNTProcessSearchInit:THandle;
begin
Result:=CreateToolHelp32snapshot(TH32CS_SNAPPROCESS,0);
end;

Function WinNTProcessSearchFirst(hSearch:THandle; Var PE:ProcessEntry32):Boolean;
begin
Result:=Process32First(hSearch,PE);
end;

Function WinNTProcessSearchNext(hSearch:THandle; Var PE:ProcessEntry32):Boolean;                                         StdCall;
begin
Result:=Process32Next(hSearch,PE);
end;

Procedure WinNTProcessSearchDestroy(hSearch:THandle);
begin
CloseHandle(hSearch);
end;

Function NtQueryInformationThread(ThreadHandle: HANDLE; ThreadInformationClass: Cardinal; ThreadInformation: PVOID; ThreadInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; stdcall;
begin
Result:=_NtQueryInformationThread(Threadhandle,ThreadInformationClass,ThreadInformation,ThreadInformationLength,ReturnLength);
end;

Function NtSetInformationThread(ThreadHandle: HANDLE; ThreadInformationClass: Cardinal; ThreadInformation: PVOID; ThreadInformationLength: ULONG): NTSTATUS; stdcall;
begin
Result:=_NtSetInformationThread(ThreadHandle,ThreadInformationClass,ThreadInformation,ThreadInformationLength);
end;

Function GetProcessID(hProcess:Cardinal):Cardinal;
Var KernelProc : Function (h:THandle):Cardinal; StdCall;
    Buffer : PPROCESS_BASIC_INFORMATION;
    Size : Cardinal;
    Status : NTSTATUS;
begin
Result:=0;
KernelProc:=GetProcAddress(GetModuleHandle('kernel32.dll'),'GetProcessId');
If Assigned(KernelProc) Then
   begin
   Result:=KernelProc(hProcess);
   If Result = 0 Then
      //ErrorMessage('GetProcessID: '+SysErrorMessage(GetLastError));
   end
Else begin
     GetMem(Buffer,SizeOf(PROCESS_BASIC_INFORMATION));
     Status:=NtQueryInformationProcess(hProcess,ProcessBasicInformation,Buffer,SizeOf(PROCESS_BASIC_INFORMATION),@Size);
     If Status = STATUS_SUCCESS Then
        Result:=Buffer.UniqueProcessID
     Else begin
          MessageBox(0,PAnsiChar(IntToHex(Int64(Status),8)),'',MB_OK Or MB_ICONERROR);
          Result:=Status;
          end;
     FreeMem(Buffer);
     end;
end;

Function NtAlertThread(ThreadHandle: HANDLE): NTSTATUS; stdcall;
begin
Result:=_NtAlertThread(ThreadHandle);
end;

Function NtAlertResumeThread(ThreadHandle: HANDLE; PreviousSuspendCount: PULONG): NTSTATUS; Stdcall;
begin
Result:=_NtAlertResumeThread(ThreadHandle,PreviousSuspendCount);
end;

Function NtQueryInformationProcess(ProcessHandle: HANDLE; ProcessInformationClass: Cardinal; ProcessInformation: PVOID; ProcessInformationLength: ULONG; ReturnLength: PULONG): NTSTATUS; stdcall;
begin
Result:=_NtQueryInformationProcess(ProcessHandle,ProcessInformationClass,ProcessInformation,ProcessInformationLength,ReturnLength);
end;

Var hLib : THandle;
Initialization
hLib:=GetModuleHandle('ntdll.dll');
If hLib > 0 Then
   begin
   _NtQueryObject:=GetProcAddress(hLib,'NtQueryObject');
   _NtQuerySystemInformation:=GetProcAddress(hLib,'NtQuerySystemInformation');
   _NtDuplicateObject:=GetProcAddress(hLib,'NtDuplicateObject');
   _NtClose:=GetProcAddress(hLib,'NtClose');
   _NtAlertThread:=GetProcAddress(hLib,'NtAlertThread');
   _NtAlertResumeThread:=GetProcAddress(hLib,'NtAlertResumeThread');
   _NtQueryInformationThread:=GetProcAddress(hLib,'NtQueryInformationThread');
   _NtSetInformationThread:=GetProcAddress(hLib,'NtSetInformationThread');
   _NtQueryInformationProcess:=GetProcAddress(hLib,'NtQueryInformationProcess');
   end;
End.
