Library ssdt;

{$R *.RES}

uses
  ShareMem,
  Windows,
  Forms,
  Classes,
  SysUtils,
  ioctls in '..\inc\ioctls.pas',
  DriversImport in '..\inc\DriversImport.pas',
  ProceduresImport in '..\inc\ProceduresImport.pas',
  Win95Functions in '..\inc\Win95Functions.pas',
  WinNTFunctions in '..\inc\WinNTFunctions.pas',
  OsVersion in '..\inc\OsVersion.pas',
  ModuleOperationImport in '..\inc\ModuleOperationImport.pas';

Var hDriver : THandle;
    FunctionNames : Array Of String;

Procedure QuerySSDTNames;
Var A : TStringList;
    I : Integer;
    Addr : Pointer;
    S    : String;
    FnName : String;
    BBuf : PByte;
    Number : Cardinal;
begin
SetLength(FunctionNames,0);
SetLength(FunctionNames,4096);
A:=GetExportListByModuleBaseAddr(GetModuleHandle('ntdll.dll'),GetCurrentProcessID);
For I:=0 To A.Count-1 Do
    begin
    S:=A[I];
    If Pos('Nt',S) = 1 Then
       begin
       FnName:=Copy(S,1,Pos('**',S)-1);
       Delete(S,1,Pos('**',S)+1);
       Addr:=Pointer(StrToInt64(Copy(S,1,Pos('**',S)-1))+GetModuleHandle('ntdll.dll'));
       BBuf:=Addr;
       If BBuf^ = $B8 Then
          begin
          Inc(BBuf);
          Number:=BBuf^;
          Inc(BBuf);
          Number:=Number+BBuf^ Shl 8;
          Inc(BBuf);
          Number:=Number+BBuf^ Shl 16;
          Inc(BBuf);
          Number:=Number+BBuf^ Shl 24;
          FunctionNames[Number]:=FnName;
          end;
       end;
    end;
end;

Procedure Block2(A:Integer); StdCall;
Var S : Cardinal;
begin
S:=0;
DeviceIoControl(hDriver,IOCTL_GMEM_X,Nil,0,Nil,0,S,Nil);
end;

Procedure Block; StdCall;
Var TID : Cardinal;
begin
CreateThread(Nil,0,@Block2,Nil,0,TID);
end;

Function GetKernelMemory(Base:Cardinal; Var Size:Cardinal; Buffer:Pointer):Cardinal; StdCall;
Var P : PKernelMemoryRecord;
   Pom : Cardinal;
begin
If Base < $80000000 Then
   begin
   Result:=STATUS_INVALID_PARAMETER;
   Exit;
   end;
GetMem(P,Size+8);
ZeroMemory(P,Size+8);
P.dwStartAddress:=Base;
P.dwSize:=Size;
If DeviceIoControl(hDriver,IOCTL_GMEM_GMEM,P,Size+8,P,Size+8,Pom,Nil) Then
   begin
   Result:=STATUS_SUCCESS;
   CopyMemory(Buffer,Pointer(Cardinal(P)+8),Pom-8);
   Size:=Pom-8;
   end
Else begin
     ErrorMessage('DeviceIoControl: '+SysErrorMessage(GetLastError));
     Result:=STATUS_UNSUCCESSFUL;
     end;
FreeMem(P,Size+8);
end;

Function GetSSDT(SSDTFunction:TSSDTRoutine):Boolean; StdCall;
Var SSDT : PSystemServiceDescriptorTable;
    Size : Cardinal;
    I : Integer;
begin
GetMem(SSDT,SizeOf(TSystemServiceDescriptorTable));
ZeroMemory(SSDT,SizeOf(TSystemServiceDescriptorTable));
Size:=SizeOf(SSDT);
Result:=False;
Result:=DeviceIoControl(hDriver,IOCTL_GMEM_GSSDT,Nil,0,SSDT,SizeOf(TSystemServiceDescriptorTable),Size,Nil);
If Result Then
   begin
   Size:=SSDT.FunctionCount;
   QuerySSDTNames;
   For I:=0 To Size-1 Do
       If Not SSDTFunction(SSDT.DescriptorBase,SSDT.DispatchBase,SSDT.FunctionCount,I,SSDT.Addresses[I],FunctionNames[I]) Then
              begin
              InformationMessage('Breaking');
              Break;
              end;
   end
Else ErrorMessage('DeviceIoControl: '+SysErrorMessage(GetLastError));
FreeMem(SSDT);
end;

Procedure LoadGMemDriver;
Var Err : Integer;
begin
InstallDriver(ExtractFilePath(Application.ExeName)+'Drivers\gmem.sys','gmem');
LoadDriver('gmem');
hDriver:=CreateFile('\\.\\gmem',GENERIC_READ Or GENERIC_WRITE,FILE_SHARE_READ Or FILE_SHARE_WRITE,Nil,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
If (Integer(hDriver) = Integer(INVALID_HANDLE_VALUE)) Or
   (hDriver = 0) Then
   begin
   ErrorMessage('CreateFile: '+SysErrorMessage(GetLastError));
   UnLoadDriver('gmem');
   UnInstallDriver('gmem');
   end;
end;

Procedure UnLoadGMemDriver;
Var A : Integer;
begin
If hDriver <> INVALID_HANDLE_VALUE Then
   begin
   CloseHandle(hDriver);
   UnLoadDriver('gmem');
   UnInstallDriver('gmem');
   end;
end;

Exports
  GetSSDT,
  GetKernelMemory,
  Block;

Function  DLLMain(dwReason:Cardinal):Boolean;
begin
Result:=True;
If Not WinNT Then
   Exit;
Case dwReason Of
     DLL_PROCESS_ATTACH : LoadGmemDriver;
     DLL_PROCESS_DETACH : UnLoadGMemDriver;
     end;
end;

Begin
DisableThreadLibraryCalls(HInstance);
DLLProc:=@DLLMain;
DLLMain(DLL_PROCESS_ATTACH);
End.
